@@ -1,7 +1,7 @@
 /*
  *  config_file.c - manages config file access.
  *	part of galculator
- *  	(c) 2002-2009 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2013 Simon Flöry (simon.floery@rechenraum.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -402,8 +402,37 @@ s_preferences config_file_read (char *filename)
 	}
 	else fprintf (stderr, _("[%s] configuration file: couldn't open configuration file %s for reading. \
 Nothing to worry about if you are starting %s for the first time. Using defaults.\n"), PACKAGE, filename, PACKAGE);
+
 	if (have_const_section == FALSE) config_file_get_default_consts (&cf_constant);
+
 	if (have_user_function_section == FALSE) config_file_get_default_user_functions (&cf_user_function);
+
+	/* If locales have changed since last start one of the separator characters
+	 * could be the new decimal point. This would lead to confusion.
+	 */
+	char decPoint = getDecPoint();
+	int resetSep = 0;
+	if (prefs.dec_sep_char[0] == decPoint) {
+		resetSep = 1;
+		prefs.dec_sep_char[0] = DEFAULT_DEC_SEP_CHAR[0];
+	}
+	if (prefs.hex_sep_char[0] == decPoint) {
+		resetSep = 1;
+		prefs.hex_sep_char[0] = DEFAULT_HEX_SEP_CHAR[0];
+	}
+	if (prefs.bin_sep_char[0] == decPoint) {
+		resetSep = 1;
+		prefs.bin_sep_char[0] = DEFAULT_BIN_SEP_CHAR[0];
+	}
+	if (prefs.oct_sep_char[0] == decPoint) {
+		resetSep = 1;
+		prefs.oct_sep_char[0] = DEFAULT_OCT_SEP_CHAR[0];
+	}
+	if (resetSep != 0) 
+		fprintf (stderr, _("[%s] configuration file - We reset at least one separator character \
+as it coincides with the decimal point. If you recently changed your locales settings, this \
+is nothing to worry about.\n"), PACKAGE);
+
 	return prefs;
 }
 
