@@ -1,7 +1,7 @@
 /*
  *  display.c - code for this nifty display.
  *	part of galculator
- *  	(c) 2002-2009 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2013 Simon Flöry (simon.floery@rechenraum.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <glade/glade.h>
 
 static GtkTextView *view;
 static GtkTextBuffer *buffer;
@@ -46,14 +45,14 @@ static char	*number_mod_labels[5] = {" DEC ", " HEX ", " OCT ", " BIN ", NULL},
 		*notation_mod_labels[4] = {" ALG ", " RPN ", " FORM ", NULL};	
 
 /* no one outside display.c should need to call these functions */
-static void display_set_line_double (double value, int line, char *tag, int number_base_status);
+static void display_set_line_double (G_REAL value, int line, char *tag, int number_base_status);
 static void display_set_line (char *string, int line, char *tag);
 static char *display_get_line (int line_nr);
 
-/* we do not need to remember the stack in double as this is stored outside
+/* we do not need to remember the stack in G_REAL as this is stored outside
  * the stack functions in here only manipulate yzt
  */
-double display_value=0.;
+G_REAL display_value=0.;
 
 /*
  * display.c mainly consists of two parts: first display setup code
@@ -73,7 +72,13 @@ gboolean on_textview_button_press_event (GtkWidget *widget,
 	char 			*selected_text;
 
 	if (event->button == 1)	{
-		gtk_widget_get_pointer (widget, &x, &y);
+        // GTK_CHECK_VERSION returns TRUE if version is at least ...
+#if GTK_CHECK_VERSION(3, 4, 0)
+        GdkModifierType mask;
+        gdk_window_get_device_position(gtk_widget_get_window(widget), gtk_get_current_event_device(), &x, &y, &mask);
+#else
+        gtk_widget_get_pointer(widget, &x, &y);
+#endif
 		gtk_text_view_get_iter_at_location (view, &start, x, y);
 		/* we return if we are in the first line */
 		if (gtk_text_iter_get_line (&start) != display_result_line+1) return FALSE;
@@ -115,13 +120,13 @@ void on_textview_selection_received (GtkWidget *widget,
 {
 	/* **** IMPORTANT **** Check to see if retrieval succeeded  */
 	/* occurs if we just press the middle button with no active selection */
-	if (data->length < 0) return;
+	if (gtk_selection_data_get_length(data) < 0) return;
 	
 	/* Make sure we got the data in the expected form */
-	if (data->type != GDK_SELECTION_TYPE_STRING) return;
+	if (gtk_selection_data_get_data_type(data) != GDK_SELECTION_TYPE_STRING) return;
 
 	/* ok, we tried to avoid this in display.* but here we can't avoid using the global var. */
-	display_result_feed ((char *)data->data, current_status.number);
+	display_result_feed ((char *)gtk_selection_data_get_data(data), current_status.number);
 
 	return;
 }
@@ -170,19 +175,20 @@ void display_create_text_tags ()
 
 void display_init ()
 {
-	GdkColor		color;
-	int			char_width;
-	PangoContext 		*pango_context;
-	PangoFontMetrics 	*font_metrics;
-	GtkTextTag		*tag;
-	PangoTabArray		*tab_array;
-	GtkTextTagTable		*tag_table;
-	
+    int                     char_width;
+    PangoContext            *pango_context;
+    PangoFontMetrics        *font_metrics;
+    GtkTextTag              *tag;
+    PangoTabArray           *tab_array;
+    GtkTextTagTable         *tag_table;
+    PangoLanguage           *language;
+    PangoFontDescription    *font_desc;
+    char                    *lang_name;
+
 	current_status.calc_entry_start_new = FALSE;
-	view = (GtkTextView *) glade_xml_get_widget (view_xml, "textview");
-	gdk_color_parse (prefs.bkg_color, &color);
-	gtk_widget_modify_base ((GtkWidget *)view, GTK_STATE_NORMAL, &color);
-	
+	view = (GtkTextView *) gtk_builder_get_object (view_xml, "textview");
+    display_set_bkg_color(prefs.bkg_color);
+
 	buffer = gtk_text_view_get_buffer (view);
 	display_create_text_tags ();
 	/* compute the approx char/digit width and create a tab stops */
@@ -190,10 +196,17 @@ void display_init ()
 	tag = gtk_text_tag_table_lookup (tag_table, "active_module");
 	/* get the approx char width */
 	pango_context = gtk_widget_get_pango_context ((GtkWidget *)view);
-	font_metrics = pango_context_get_metrics(pango_context, \
-		tag->values->font,
-		tag->values->language);
-	char_width = MAX (pango_font_metrics_get_approximate_char_width (font_metrics), \
+
+    g_object_get(tag, "font-desc", &font_desc, "language", &lang_name, NULL);
+    language = pango_language_from_string(lang_name);
+    g_free(lang_name);
+	font_metrics = pango_context_get_metrics(pango_context,
+		font_desc,
+		language);
+    pango_font_description_free(font_desc);
+    g_boxed_free(PANGO_TYPE_LANGUAGE, language);
+
+	char_width = MAX (pango_font_metrics_get_approximate_char_width (font_metrics),
 		pango_font_metrics_get_approximate_digit_width (font_metrics));
 	tab_array = pango_tab_array_new_with_positions (1, FALSE, PANGO_TAB_LEFT, 3*char_width);
 	gtk_text_view_set_tabs (view, tab_array);
@@ -542,14 +555,24 @@ void display_module_base_delete (char *mark_name, char **text)
 
 void display_change_option (int old_status, int new_status, int opt_group)
 {
-	double	display_value=0;
-	double 	*stack;
+	G_REAL	display_value=0;
+	G_REAL 	*stack;
+	int 	counter;
 	
 	switch (opt_group) {
 		case DISPLAY_OPT_NUMBER:
 			update_active_buttons (new_status, current_status.notation);
 			display_value = display_result_get_double (old_status);
 			stack = display_stack_get_yzt_double (old_status);
+			/* In case we convert from decimal to integer, we round display 
+			 * values to integers. As in ftoax we use G_FLOOR, we use it here as
+			 * well.
+			 */
+			if ((old_status == CS_DEC) && (new_status != CS_DEC)) {
+				display_value = G_FLOOR(display_value);
+				for (counter = 0; counter < display_result_line; counter++)
+					stack[counter] = G_FLOOR(stack[counter]);
+			}
 			display_result_set_double (display_value, new_status);
 			display_stack_set_yzt_double (stack, new_status);
 			g_free (stack);
@@ -582,11 +605,20 @@ void display_change_option (int old_status, int new_status, int opt_group)
 
 void display_set_bkg_color (char *color_string)
 {
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GdkRGBA	color;
+	if (prefs.mode == PAPER_MODE) return;
+	gdk_rgba_parse (&color, color_string);
+	if (view)
+        gtk_widget_override_background_color(GTK_WIDGET(view), 
+            GTK_STATE_FLAG_NORMAL, &color);
+#else
 	GdkColor	color;
-	
 	if (prefs.mode == PAPER_MODE) return;
 	gdk_color_parse (color_string, &color);
-	if (view) gtk_widget_modify_base ((GtkWidget *)view, GTK_STATE_NORMAL, &color);
+	if (view)
+        gtk_widget_modify_base ((GtkWidget *)view, GTK_STATE_NORMAL, &color);
+#endif
 }
 
 /*
@@ -662,7 +694,7 @@ static void display_set_line (char *string, int line, char *tag)
 	}
 }
 
-static void display_set_line_double (double value, int line, char *tag, int number_base_status)
+static void display_set_line_double (G_REAL value, int line, char *tag, int number_base_status)
 {
 	char		*string_value;
 	
@@ -714,7 +746,7 @@ void display_result_add_digit (char digit, int number_base_status)
  * display_result_set_double. set text of calc_entry to string given by float value.
  *	the float value is manipulated (rounded, ...)
  */
-void display_result_set_double (double value, int number_base_status)
+void display_result_set_double (G_REAL value, int number_base_status)
 {	
 	current_status.allow_arith_op = TRUE;
 	display_module_arith_label_update (' ');
@@ -722,7 +754,7 @@ void display_result_set_double (double value, int number_base_status)
 	display_value = value;
 }
 
-void display_result_set (char *string_value, int update_display_value, double value)
+void display_result_set (char *string_value, int update_display_value, G_REAL value)
 {
 	current_status.allow_arith_op = TRUE;
 	display_module_arith_label_update (' ');
@@ -734,14 +766,35 @@ void display_result_set (char *string_value, int update_display_value, double va
 void display_result_feed (char *string, int number_base_status)
 {
 	int	counter;
-	
-	for (counter = 0; counter < strlen(string); counter++) {
-		if (is_valid_number(current_status.number, string[counter])) 
-			/* g_ascii_toupper to convert lower hex chars to upper */
-			display_result_add_digit (
-				g_ascii_toupper(string[counter]), number_base_status);
+
+    gboolean toggleSign = current_status.calc_entry_start_new && (strlen(string) > 0) && (*string == '-');
+
+    for (counter = 0; counter < strlen(string); counter++) {
+        switch (string[counter]) {
+        case '-': {
+            /* this only applies if we just had "EE" before */
+            char *dline = display_result_get();
+            if (strlen(dline) < 3) break;
+            if (strncmp(&dline[strlen(dline)-2], "e+", 2) == 0) display_result_toggle_sign(NULL);
+            break;
+        }
+        case 'e':
+        case 'E':
+			/* In decimal mode, call display_append_e, otherwise, run into 
+			 * default handler. simon20130214
+			 */
+			if (current_status.number == CS_DEC) {
+				display_append_e(NULL);
+				break;
+			}
+        default:
+            /* g_ascii_toupper to convert lower hex chars to upper */
+            if (is_valid_number(current_status.number, string[counter]))
+                display_result_add_digit(g_ascii_toupper(string[counter]), number_base_status);
+        }
+        if (toggleSign && counter==1) display_result_toggle_sign(NULL);
 	}
-	if (string[0] == '-') display_result_toggle_sign (NULL);
+//
 }
 
 /*
@@ -766,12 +819,12 @@ char *display_result_get ()
 	return display_get_line (display_result_line);
 }
 
-/* display_result_get_double. a display_result_get plus conversion to double
+/* display_result_get_double. a display_result_get plus conversion to G_REAL
  */
-double display_result_get_double (int number_base_status)
+G_REAL display_result_get_double (int number_base_status)
 {
 /*	char 	*result_string;
-	double	ret_val;
+	G_REAL	ret_val;
 	
 	result_string = display_result_get();
 	ret_val = string2double (result_string, number_base_status);
@@ -844,7 +897,7 @@ void display_result_backspace (int number_base_status)
 		current_entry = display_result_get();
 		/* to avoid an empty/senseless result field */
 		if (strlen(current_entry) == 1) current_entry[0] = '0';
-		else if ((strlen(current_entry) == 2) && (*current_entry == '-')) current_entry = "0\0";
+        else if ((strlen(current_entry) == 2) && (*current_entry == '-')) strcpy(current_entry, "0");
 		else if (current_entry[strlen(current_entry) - 2] == 'e') current_entry[strlen(current_entry) - 2] = '\0';
 		else current_entry[strlen(current_entry) - 1] = '\0';
 		display_result_set (current_entry, TRUE, string2double (current_entry, number_base_status));
@@ -903,7 +956,7 @@ void display_stack_remove ()
  *	calc_basic IS NOT modified.
  */
 
-void display_stack_set_yzt_double (double *stack, int number_base_status)
+void display_stack_set_yzt_double (G_REAL *stack, int number_base_status)
 {
 	int		counter;
 
@@ -949,18 +1002,18 @@ char **display_stack_get_yzt ()
 
 /*
  * display_stack_get_yzt_double - returns the string array of display_stack_get_yzt
- * 	as double array.
+ * 	as G_REAL array.
  */
 
-double *display_stack_get_yzt_double (int number_base_status)
+G_REAL *display_stack_get_yzt_double (int number_base_status)
 {
 	char 	**string_stack;
-	double	*double_stack;
+	G_REAL	*G_REAL_stack;
 	int	counter;
 	
-	double_stack = (double *) malloc (display_result_line * sizeof(double));
+	G_REAL_stack = (G_REAL *) malloc (display_result_line * sizeof(G_REAL));
 	string_stack = display_stack_get_yzt();
 	for (counter = 0; counter < display_result_line; counter++)
-		double_stack[counter] = string2double (string_stack[counter], number_base_status);
-	return double_stack;
+		G_REAL_stack[counter] = string2double (string_stack[counter], number_base_status);
+	return G_REAL_stack;
 }
