@@ -1,7 +1,7 @@
 /*
  *  general_functions.c - this and that.
  *	part of galculator
- *  	(c) 2002-2009 Simon Floery (chimaira@users.sf.net)
+ *  	(c) 2002-2013 Simon Flöry (simon.floery@rechenraum.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
+#include <locale.h>
 
 #include "galculator.h"
 #include "general_functions.h"
@@ -36,15 +37,14 @@
 #include "flex_parser.h"
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
-double error_unsupported_inv (double dummy)
+G_REAL error_unsupported_inv (G_REAL dummy)
 {
 	error_message (_("unsupported inverse function"));
 	return dummy;
 }
 
-double error_unsupported_hyp (double dummy)
+G_REAL error_unsupported_hyp (G_REAL dummy)
 {
 	error_message (_("unsupported hyperbolic function"));
 	return dummy;
@@ -118,9 +118,9 @@ void all_clear ()
  *  a number is called negative, if its msb is set!
  */
 
-double axtof (char *bin_string, int base, int nr_bits, gboolean is_signed)
+G_REAL axtof (char *bin_string, int base, int nr_bits, gboolean is_signed)
 {
-	double 		return_value=0;
+	G_REAL 		return_value=0;
 	int		counter;
 	char		*lower_bin_string;
 	
@@ -130,9 +130,9 @@ double axtof (char *bin_string, int base, int nr_bits, gboolean is_signed)
 
 	for (counter = strlen (lower_bin_string) - 1; counter >= 0; counter--) {
 		if (lower_bin_string[counter] - '0' < 10) \
-			return_value += (lower_bin_string[counter] - '0') * pow (base, strlen (lower_bin_string) - 1 - counter);
+			return_value += (lower_bin_string[counter] - '0') * G_POW (base, strlen (lower_bin_string) - 1 - counter);
 		else if (lower_bin_string[counter] - 'a' < 10) \
-			return_value += (lower_bin_string[counter] - 'a' + 10) * pow (base, strlen (lower_bin_string) - 1 - counter);
+			return_value += (lower_bin_string[counter] - 'a' + 10) * G_POW (base, strlen (lower_bin_string) - 1 - counter);
 		else fprintf (stderr, _("[%s] failed to convert char %c in function \"axtof\". %s\n"), PROG_NAME, lower_bin_string[counter], BUG_REPORT);
 	}
 	
@@ -142,16 +142,16 @@ double axtof (char *bin_string, int base, int nr_bits, gboolean is_signed)
 
 	/* if most significant bit is set, its a negative number. using */
 	if (is_signed == TRUE) 
-		if (return_value >= pow (2, nr_bits - 1))
-			return_value = (-1) * (pow (2, nr_bits) - return_value);
+		if (return_value >= G_POW (2, nr_bits - 1))
+			return_value = (-1) * (G_POW (2, nr_bits) - return_value);
 	return return_value;
 }
 
 /* rem: my own remainder function to deal with precision problems */
 
-int rem (double x, long long int y)
+int rem (G_REAL x, G_HUGEINT y)
 {
-	return x - ((long long int)(x/((double)y)))*y;
+	return x - G_FLOOR(x / (G_REAL)y) * y;
 }
 
 /* ftoax - number to string conversion.
@@ -159,25 +159,25 @@ int rem (double x, long long int y)
  *  see axtof for details about what is a negative number.
  */
 
-char *ftoax (double x, int base, int nr_bits, gboolean is_signed)
+char *ftoax (G_REAL x, int base, int nr_bits, gboolean is_signed)
 {
 	char		*return_string;
 	int		length=0, counter, remainder;
-	double		localx;
+	G_REAL		localx;
 	
 	/* handle huge values --> infinity */
 	if (is_signed == TRUE) {
-		if (x < (-1)*pow (2, nr_bits - 1)) return g_strdup(MY_INFINITY_STRING);
-		if (x >= pow (2, nr_bits - 1)) return g_strdup(MY_INFINITY_STRING);
+		if (x < (-1)*G_POW (2, nr_bits - 1)) return g_strdup(MY_INFINITY_STRING);
+		if (x >= G_POW (2, nr_bits - 1)) return g_strdup(MY_INFINITY_STRING);
 		/* handle negative numbers */
-		if (x < 0) x = pow (2, nr_bits) + x;
+		if (x < 0) x = G_POW (2, nr_bits) + x;
 	} else {
-		if (x >= pow (2, nr_bits)) return g_strdup(MY_INFINITY_STRING);
+		if (x >= G_POW (2, nr_bits)) return g_strdup(MY_INFINITY_STRING);
 		if (x < 0) return g_strdup(MY_INFINITY_STRING);
 	}
 	/* doing it this way and not with logs as this is much more numerical stable */
 	localx = x;
-	while ((localx=floor(localx/(double)base)) >= 1) length++;
+	while ((localx=G_FLOOR(localx/(G_REAL)base)) >= 1) length++;
 	length+=2;
 	
 	localx = x;
@@ -187,8 +187,12 @@ char *ftoax (double x, int base, int nr_bits, gboolean is_signed)
 		remainder = rem (localx, base);
 		if (remainder < 10) return_string[counter] = '0' + remainder;
 		else if (remainder < 20) return_string[counter] = 'A' + remainder - 10;
-		else fprintf (stderr, _("[%s] failed to convert %f in function \"ftoax\". %s\n"), PROG_NAME, x, BUG_REPORT);
-		localx = floor (localx / (double)base);
+		else {
+			char *sx = float2string("%"G_LMOD"f", x);
+			fprintf (stderr, _("[%s] failed to convert %s in function \"ftoax\". %s\n"), PROG_NAME, sx, BUG_REPORT);
+			free(sx);
+		}
+		localx = G_FLOOR (localx / (G_REAL)base);
 	}
 	return return_string;
 }
@@ -217,23 +221,23 @@ char *add_leading_zeros (char *string, int multiple)
  * preference dialog sets. these are handler for the big configuration struct.
  */
 
-void set_button_font (GladeXML *xml, char *button_name, void *new_label)
+void set_button_font (GtkBuilder *xml, char *button_name, void *new_label)
 {
 	GtkButton	*button;
 	char 		**string_var;
 	
 	string_var = new_label;	
-	button = (GtkButton *) glade_xml_get_widget (xml, button_name);
+	button = (GtkButton *) gtk_builder_get_object (xml, button_name);
 	if (button) gtk_font_button_set_font_name (GTK_FONT_BUTTON(button), *string_var);	
 }
 
-void set_checkbutton (GladeXML *xml, char *checkbutton_name, void *is_active)
+void set_checkbutton (GtkBuilder *xml, char *checkbutton_name, void *is_active)
 {
 	GtkToggleButton		*toggle_button;
 	gboolean		*bool_var;
 	
 	bool_var = is_active;
-	toggle_button = (GtkToggleButton *) glade_xml_get_widget (xml, checkbutton_name);	
+	toggle_button = (GtkToggleButton *) gtk_builder_get_object (xml, checkbutton_name);	
 	if (toggle_button) {
 		gtk_toggle_button_set_active (toggle_button, *bool_var);
 		/* they say there is no good reason. i say there is */
@@ -241,45 +245,41 @@ void set_checkbutton (GladeXML *xml, char *checkbutton_name, void *is_active)
 	}
 }
 
-void set_spinbutton (GladeXML *xml, char *spinbutton_name, void *value)
+void set_spinbutton (GtkBuilder *xml, char *spinbutton_name, void *value)
 {
 	GtkSpinButton	*spin_button;
 	int		*int_var;
 	
 	int_var = value;
-	/*d_var = (double) *int_var;*/
-	spin_button = (GtkSpinButton *) glade_xml_get_widget (xml, spinbutton_name);
+	/*d_var = (G_REAL) *int_var;*/
+	spin_button = (GtkSpinButton *) gtk_builder_get_object (xml, spinbutton_name);
 	if (spin_button) gtk_spin_button_set_value (spin_button, *int_var);
 }
 
-void set_optmenu (GladeXML *xml, char *optmenu_name, void *index)
-{
-	GtkOptionMenu	*opt_menu;
-	int		*int_var;
-	
-	int_var = index;	
-	opt_menu = (GtkOptionMenu *) glade_xml_get_widget (xml, optmenu_name);
-	if (opt_menu) gtk_option_menu_set_history (opt_menu, *int_var);
-}
-
 /*
  *
  */
 
-void set_button_color (GladeXML *xml, char *button_name, void *color_string)
+void set_button_color (GtkBuilder *xml, char *button_name, void *color_string)
 {
-	GtkWidget	*button;
-	char 		**string_var;
-	GdkColor	color;
-
-	/* dereference */
-	string_var = color_string;
-	gdk_color_parse (*string_var, &color);
-	button = glade_xml_get_widget (xml, button_name);
-	if (button) gtk_color_button_set_color (GTK_COLOR_BUTTON(button), &color);
+	GtkWidget *button = GTK_WIDGET(gtk_builder_get_object (xml, button_name));
+	char **string_var = color_string;
+		
+	if (button) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+		GdkRGBA color;
+		if (!gdk_rgba_parse(&color, *string_var))
+			fprintf (stderr, _("[%s] failed to convert color %s in function \"set_button_color\". %s\n"), PROG_NAME, *string_var, BUG_REPORT);
+		gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(button), &color);
+#else
+		GdkColor color;
+		gdk_color_parse(*string_var, &color);
+		gtk_color_button_set_color (GTK_COLOR_BUTTON(button), &color);
+#endif
+	}
 }
 
-void set_stacksize (GladeXML *xml, char *name, void *stack_size)
+void set_stacksize (GtkBuilder *xml, char *name, void *stack_size)
 {
 	int		*size;
 	GtkToggleButton	*tb;
@@ -287,25 +287,25 @@ void set_stacksize (GladeXML *xml, char *name, void *stack_size)
 	/* name is NULL */
 	size = stack_size;
 	if (*size == RPN_FINITE_STACK)
-		tb = (GtkToggleButton *) glade_xml_get_widget (xml, "finite_stack_size");
-	else tb = (GtkToggleButton *) glade_xml_get_widget (xml, "infinite_stack_size");
+		tb = (GtkToggleButton *) gtk_builder_get_object (xml, "finite_stack_size");
+	else tb = (GtkToggleButton *) gtk_builder_get_object (xml, "infinite_stack_size");
 	gtk_toggle_button_set_active (tb, TRUE);
 }
 
-void set_entry (GladeXML *xml, char *entry_name, void *entry_text)
+void set_entry (GtkBuilder *xml, char *entry_name, void *entry_text)
 {
 	GtkEntry	*entry;
 	char 		**string_var;
 
 	string_var = entry_text;	
-	entry = (GtkEntry *) glade_xml_get_widget (xml, entry_name);
+	entry = (GtkEntry *) gtk_builder_get_object (xml, entry_name);
 	if (entry) gtk_entry_set_text (entry, *string_var);	
 }
 
 /*
  * convert given GdkColor to a string so that gdk_color_parse gives the 
  * same color again. Formerly called gdk_color_to_string but since version 2.12
- * gdk features that function by itself.
+ * gdk features that function by itself. For GTK3, we use gdk_rgba.
  */
 
 char *convert_gdk_color_to_string (GdkColor color)
@@ -321,33 +321,24 @@ void apply_preferences (s_preferences prefs)
 {
 	GtkWidget	*menu_item;
 
-	/* not view specific */
-#ifdef WITH_HILDON
-	set_widget_visibility (main_window_xml, "main_menu", prefs.show_menu);
-#else
- 	set_widget_visibility (main_window_xml, "menubar", prefs.show_menu);
-#endif
-	menu_item = glade_xml_get_widget (main_window_xml, "show_menubar1");
-	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menu_item, prefs.show_menu);
+    menu_item = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "show_menubar1"));
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), prefs.show_menu);
 
 	switch (prefs.mode) {
 	case BASIC_MODE:
-		menu_item = 
-			glade_xml_get_widget (main_window_xml, "basic_mode");
+        menu_item = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "basic_mode"));
 		break;
 	case SCIENTIFIC_MODE:
-		menu_item = 
-			glade_xml_get_widget (main_window_xml, "scientific_mode");
+        menu_item = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "scientific_mode"));
 		break;
 	case PAPER_MODE:
-		menu_item = 
-			glade_xml_get_widget (main_window_xml, "paper_mode");
+        menu_item = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "paper_mode"));
 		break;
 	default:
 		error_message ("Unknown mode %i in \"apply_preferences\"", prefs.mode);
 	}
-	gtk_menu_item_activate ((GtkMenuItem *) menu_item);
-	
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+
 	/* view specific */
 /*	display_update_tags ();
 	display_set_bkg_color (prefs.bkg_color);
@@ -361,19 +352,6 @@ void apply_preferences (s_preferences prefs)
 */
 }
 
-/*
- * this function changes the foreground color for all states to the given string
- */
-
-void gtk_widget_really_modify_fg (GtkWidget *widget, GdkColor color)
-{
-	gtk_widget_modify_fg (widget, GTK_STATE_NORMAL, &color);
-	gtk_widget_modify_fg (widget, GTK_STATE_ACTIVE, &color);
-	gtk_widget_modify_fg (widget, GTK_STATE_PRELIGHT, &color);
-	gtk_widget_modify_fg (widget, GTK_STATE_SELECTED, &color);
-	gtk_widget_modify_fg (widget, GTK_STATE_INSENSITIVE, &color);
-}
-
 gboolean is_valid_number (int number_base, char number)
 {
 	char *valid_numbers[4]={"1234567890", "1234567890abcdef", "12345670", "01"};
@@ -389,15 +367,25 @@ gboolean is_valid_number (int number_base, char number)
 void activate_menu_item (char *item_name)
 {
 	GtkMenuItem		*menu_item;
-	
-	menu_item = (GtkMenuItem *) glade_xml_get_widget (main_window_xml, \
-		g_strstrip (g_ascii_strdown (item_name, -1)));
+    char* lower_item_name = g_ascii_strdown (item_name, -1);
+
+	menu_item = (GtkMenuItem *) gtk_builder_get_object (main_window_xml,
+		g_strstrip (lower_item_name));
+    g_free(lower_item_name);
 	if (menu_item)
-	/* as we use this only for menu boxes, a simple activate is enough.
-	 * the extra magic in src/callbacks.c::on_scientific_mode_activate
-	 * is necessary only for checkboxmenuitems.
-	 */
-		gtk_menu_item_activate ((GtkMenuItem *) menu_item);
+    {
+        if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item)) == TRUE)
+        {
+            /* PCMan: if the checkbox is already checked, gtk+ won't emit
+             * a "toggled" signal again if we call set_active().
+             * so let's trigger "toggled" signal manually since galculator
+             * relies heavily on this behavior and cannot work without it.
+             * FIXME: This is not the right way to use gtk+, though. */
+            gtk_check_menu_item_toggled(GTK_CHECK_MENU_ITEM(menu_item));
+        }
+        else
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+    }
 	else fprintf (stderr, _("[%s] failed to find widget %s in function \"activate_menu_item\". %s\n"), PROG_NAME, item_name, BUG_REPORT);
 }
 
@@ -405,7 +393,7 @@ void activate_menu_item (char *item_name)
  *	string should be freed.
  */
 
-char *get_display_number_string (double value, int base)
+char *get_display_number_string (G_REAL value, int base)
 {
 	char 	*string_value, *string_value0, *string_value1;
 	
@@ -417,8 +405,8 @@ char *get_display_number_string (double value, int base)
 			 * different precision levels and check whether one result is
 			 * significantly shorter than the other.
 			 */
-			string_value0 = g_strdup_printf("%.*g", get_display_number_length(current_status.number) - 1, value);
-			string_value1 = g_strdup_printf("%.*g", get_display_number_length(current_status.number), value);
+			string_value0 = float2stringP("%.*"G_LMOD"g", get_display_number_length(current_status.number) - 1, value);
+			string_value1 = float2stringP("%.*"G_LMOD"g", get_display_number_length(current_status.number), value);
 			if (strlen(string_value0) + 1 < strlen(string_value1)) {
 				string_value = string_value0;
 				g_free(string_value1);
@@ -478,7 +466,7 @@ int get_display_number_length (int base)
 
 void gfunc_f1 (GtkToggleButton *button)
 {
-	double		*stack;
+	G_REAL		*stack;
 	
 	if (current_status.notation == CS_PAN) 
 		on_operation_button_clicked (button, NULL);
@@ -498,7 +486,7 @@ void gfunc_f1 (GtkToggleButton *button)
 
 void gfunc_f2 (GtkToggleButton *button)
 {
-	double 		*stack;
+	G_REAL 		*stack;
 	
 	if (current_status.notation == CS_PAN)
 		on_operation_button_clicked (button, NULL);
@@ -523,7 +511,7 @@ void gfunc_f2 (GtkToggleButton *button)
 
 void rpn_stack_lift ()
 {
-	double	*stack;
+	G_REAL	*stack;
 	
 	if ((current_status.notation == CS_RPN) && 
 		(current_status.rpn_stack_lift_enabled == TRUE)) {
@@ -539,7 +527,7 @@ void remember_display_values()
 {
 	char 	*stack[3];
 	
-	if (prefs.rem_display == TRUE) {
+    if (prefs.rem_display == TRUE) {
 		display_result_set (prefs.rem_valuex, TRUE, string2double(prefs.rem_valuex, current_status.number));
 		/* for the result setting the display string is enough */
 		if (current_status.notation == CS_RPN) {
@@ -556,16 +544,16 @@ void remember_display_values()
 }
 
 /*
- * string2double - this function makes a string to double conversion with 
+ * string2double - this function makes a string to G_REAL conversion with 
  * 	respect to supplied number base. if number base < 0 we try to 
  *	determine from string which should have then a 0? prefix.
  *	it uses axtof.
  */
 
-double string2double (char *string, int number_base)
+G_REAL string2double (char *string, int number_base)
 {
 	char 	*end_ptr;
-	double	ret_val;
+	G_REAL	ret_val;
 	
 	switch (number_base) {
 		case CS_DEC:
@@ -672,18 +660,13 @@ char *string_del_separator (char *string, char separator)
 	return string;
 }
 
-void set_button_label_and_tooltip (GladeXML *xml, char *button_name, 
+void set_button_label_and_tooltip (GtkBuilder *xml, char *button_name, 
 	char *label, char *tooltip)
 {
-	GtkWidget	*w;
-	GtkTooltipsData	*tooltip_data;
-	
-	w = glade_xml_get_widget (xml, button_name);
+    GtkWidget *w = GTK_WIDGET(gtk_builder_get_object (xml, button_name));
 	if (w) {
 		gtk_button_set_label ((GtkButton *)w, label);
-		
-		tooltip_data = gtk_tooltips_data_get (w);
-		gtk_tooltips_set_tip(tooltip_data->tooltips, w, tooltip, tooltip_data->tip_private);
+        gtk_widget_set_tooltip_text(w, tooltip);
 	}
 }
 
@@ -691,12 +674,12 @@ GtkWidget *formula_entry_is_active (GtkWidget *window_widget)
 {
 	GtkWidget	*active_widget=NULL, *main_window=NULL;
 	
-	main_window = glade_xml_get_widget (main_window_xml, "main_window");
+    main_window = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "main_window"));
 	if (main_window != NULL)
-		active_widget = gtk_window_get_focus ((GtkWindow *)main_window);
+        active_widget = GTK_WIDGET(gtk_window_get_focus ((GtkWindow *)main_window));
 	if (active_widget != NULL)
-		if ((strcmp (gtk_widget_get_name (gtk_widget_get_toplevel(window_widget)),"main_window") == 0) &&
-			(strcmp (gtk_widget_get_name (active_widget), "formula_entry") == 0)) 
+		if ((g_strcmp0 (gtk_buildable_get_name (GTK_BUILDABLE(gtk_widget_get_toplevel(window_widget))),"main_window") == 0) &&
+			(g_strcmp0 (gtk_buildable_get_name (GTK_BUILDABLE(active_widget)), "formula_entry") == 0)) 
 			return active_widget;
 	return NULL;
 }
@@ -709,11 +692,11 @@ GtkWidget *formula_entry_is_active_no_toplevel_check ()
 {
 	GtkWidget	*active_widget=NULL, *main_window=NULL;
 	
-	main_window = glade_xml_get_widget (main_window_xml, "main_window");
+    main_window = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "main_window"));
 	if (main_window) 
-		active_widget = gtk_window_get_focus ((GtkWindow *) main_window);
+        active_widget = GTK_WIDGET(gtk_window_get_focus ((GtkWindow *) main_window));
 	if (active_widget)
-		if (strcmp (gtk_widget_get_name (active_widget), "formula_entry") == 0) 
+		if (g_strcmp0 (gtk_buildable_get_name (GTK_BUILDABLE(active_widget)), "formula_entry") == 0) 
 			return active_widget;
 	return NULL;
 }
@@ -739,7 +722,7 @@ s_flex_parser_result compute_user_function (char *expression, char *variable, ch
 	return result;
 }
 
-double x2rad (double x)
+G_REAL x2rad (G_REAL x)
 {
 	switch (current_status.angle){
 	case CS_DEG:
@@ -754,7 +737,7 @@ double x2rad (double x)
 	return x;
 }	
 
-double rad2x (double rad)
+G_REAL rad2x (G_REAL rad)
 {
 	switch (current_status.angle) {
 	case CS_DEG:
@@ -894,9 +877,39 @@ void change_option (int new_status, int opt_group)
  */
 void set_window_size_minimal()
 {
-	GtkWidget	*main_window=NULL;
+    GtkWidget *main_window = GTK_WIDGET(gtk_builder_get_object (main_window_xml, "main_window"));
+	if (main_window != NULL) {
+        gtk_window_resize ((GtkWindow *)GTK_WIDGET(gtk_widget_get_toplevel(main_window)), 1, 1);
+#if (GTK_CHECK_VERSION(3, 0, 0))
+		/* with gtk 3.6.0 on ubuntu I observed that simply calling gtk_window_resize
+		 * with (1,1) did not properly resize widgets and the main window. 
+		 * Interestingly, performing the subsequent query solved the problem in
+		 * almost all cases. I assume this being a bug in GTK which I do not
+		 * pursue any further, until I get told to so.
+		 */
+		gtk_widget_get_preferred_size(GTK_WIDGET(gtk_widget_get_toplevel(main_window)), NULL, NULL);
+#endif
+	}
+}
+
+/*! \brief Determine decimal point from locale settings 
+ *
+ * If locale's decimal point takes more than a single character, we fallback
+ * to the DEFAULT_DEC_POINT.
+ * 
+ * \return Decimal point
+ */
+char getDecPoint()
+{
+	struct lconv 	*locale_settings;
 	
-	main_window = glade_xml_get_widget (main_window_xml, "main_window");
-	if (main_window != NULL)
-		gtk_window_resize ((GtkWindow *)gtk_widget_get_toplevel(main_window), 1, 1);
+	char decPoint = DEFAULT_DEC_POINT;
+	locale_settings = localeconv();
+	if (strlen (locale_settings->decimal_point) != 1) {
+		fprintf (stderr, _("[%s] length of decimal point (in locale) \
+is not supported: >%s<\nYou might face problems when using %s! %s\n)"), 
+		PACKAGE, locale_settings->decimal_point, PROG_NAME, BUG_REPORT);
+	} else decPoint = locale_settings->decimal_point[0];
+	
+	return decPoint;
 }
